﻿using System;
using System.Collections.Generic;
using System.Linq;
using TestCoverageAnalysisTool;

namespace TestCoverageAnalysisTool
{
    public class Tree
    {
        private TreeNode _root = null;
        private List<TreeNode> _nodes = new List<TreeNode>();
        private Dictionary<NodeName, TreeNode> _keys = new Dictionary<NodeName, TreeNode>();

        

        public List<TreeNode> Nodes
        {
            get
            {
                return _nodes;
            }
        }

        public Dictionary<NodeName, TreeNode> Keys
        {
            get
            {
                return _keys;
            }
        }

        public Tree()
        {
            
        }

        public Matrix GetAdjMatrix(out NodeNameCollection contents)
        {
            var adjMatrix = new Matrix(_nodes.Count);
            contents = new NodeNameCollection();

            for (int i = 0; i < _nodes.Count; i++)
            {
                var node = _nodes[i];

                contents.Add(node.Name);

                foreach (var child in node.Children)
                {
                    var childIndex = _nodes.IndexOf(child);

                    adjMatrix[i, childIndex] = 1;
                }
            }

            return adjMatrix;
        }

        public Tree(Matrix matrix, NodeNameCollection contents)
        {
            var distNodes = contents.Items.Distinct().ToList();

            Root = new TreeNode(contents[0]);

            _keys.Add(contents[0], Root);

            Root.OwnerTree = this;

            for (int i = 1; i < matrix.Dimension; i++)
            {
                var content = contents[i];

                var node = new TreeNode(content);
                node.OwnerTree = this;

                _nodes.Add(node);
                _keys.Add(content, node);
            }

            for (int i = 0; i < matrix.Dimension; i++)
            {
                for (int j = 0; j < matrix.Dimension; j++)
                {
                    if (matrix[i, j] == 1)
                    {
                        _nodes[i].Children.Add(_nodes[j]);
                    }
                }
            }
        }

        public TreeNode this[NodeName key]
        {
            get
            {
                return _keys[key];
            }
        }

        public TreeNode Root
        {
            get
            {
                return _root;
            }
            set
            {
                _root = value;
                _nodes.Add(_root);
                _root.Children.ItemAdded += Children_ItemAdded;
            }
        }

        void Children_ItemAdded(TreeNode obj)
        {
            // TODO: Неэффективно!
            if (!_nodes.Contains(obj))
            {
                _nodes.Add(obj);
            }
        }
    }
}