﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace TestCoverageAnalysisTool
{
    public class TreeNode
    {
        private HashSet<TreeNode> _parents;

        public Int32 Depth { get; set; }

        public TreeNode()
        {
            _parents = new HashSet<TreeNode>();

            Children = new ObservableSet<TreeNode>();
            Children.ItemAdded += Children_ItemAdded;
        }

        public TreeNode(NodeName content) : this()
        {
            Name = content;
        }

        void Children_ItemAdded(TreeNode obj)
        {
            obj._parents.Add(this);
        }

        public Tree OwnerTree { get; set; }

        /// <summary>
        /// Содержимое узла
        /// </summary>
        public NodeName Name { get; set; }

        /// <summary>
        /// Коллекция родительских узлов.
        /// </summary>
        public ReadOnlyCollection<TreeNode> Parents
        {
            get
            {
                return new ReadOnlyCollection<TreeNode>(_parents.ToList());
            }
        }

        public void ReplaceByTree(Tree tree, bool cutRoot = true)
        {
            // Находим выходной узел 

            TreeNode exitNode = null;

            foreach (var treeNode in tree.Nodes)
            {
                if (treeNode.Children.Count == 0)
                {
                    exitNode = treeNode;
                    break;
                }
            }

            // Находим входной узел в присоединяемом дереве. Предполагаем, что от корня нет веток.

            var firstInSecondTree = tree.Root.Children.First();

            tree.Root.Children.Clear();
            firstInSecondTree._parents.Clear();

            // Родителям замещаемого узла меняем ссылку на потомка на первый узел присоединяемого дерева.

            foreach (var treeNode in Parents)
            {
                treeNode.Children.Remove(this);
                treeNode.Children.Add(firstInSecondTree);
            }
            
            // Потомку замещаемого узла меняем родителей на последние узлы присоединяемого дерева.

            var nextNode = this.Children.First();

            this.Children.Remove(nextNode);

            nextNode._parents.Remove(this);

            foreach (var treeNode in exitNode.Parents)
            {
                nextNode._parents.Add(treeNode);
                treeNode.Children.Add(nextNode);
                treeNode.Children.Remove(exitNode);
            }
            
            var curNodeIndex = OwnerTree.Nodes.IndexOf(this);

            int inserted = 0;

            foreach (var treeNode in tree.Nodes)
            {
                if ((cutRoot && treeNode == tree.Root) || (cutRoot && treeNode == exitNode))
                {
                    continue;
                }
                // TODO: У узла остается связь с деревом (занимает память)
                treeNode.OwnerTree = OwnerTree;

                treeNode.Depth++;

                OwnerTree.Nodes.Insert(curNodeIndex+inserted+1, treeNode);
                OwnerTree.Keys.Add(treeNode.Name, treeNode);

                inserted++;
            }
        }

        /// <summary>
        /// Коллекция дочерних узлов.
        /// </summary>
        public ObservableSet<TreeNode> Children { get; set; }

        public override string ToString()
        {
            return String.Format("={0}=", Name);
        }
    }
}